package com.smith9.geiger.client

import com.smith9.geiger.client.beans.Duration
import org.apache.commons.httpclient.methods.PostMethod
import org.apache.commons.httpclient.HttpClient

import org.joda.time.DateTime

import org.slf4j.Logger
import org.slf4j.LoggerFactory
import com.smith9.geiger.client.beans.Stat
import com.smith9.geiger.client.beans.Count
import com.smith9.geiger.client.beans.Period

/**
 * Created: Mar 17, 2011
 * @author david
 */

class GeigerClient implements LoaderClient {
    private static final Logger LOG = LoggerFactory.getLogger(GeigerClient.class)
    private URL url
    private long totalSent
    private long totalDuration
//    private long batchesSent
    private HttpClient client

    public static void main(String[] args) {
        boolean loadDurations = true
        if (loadDurations) {
            List<Duration> durations = [
                new Duration(environment: "SOTL", product: "EasyDoc", subProduct: "PI", startTime: new DateTime(), duration: 2, measure: "end to end", grouping: "1234567", reference: "123:1", failure: false, metadata: [start: "1", end: "5"]),
                new Duration(environment: "SOTL", product: "EasyDoc", subProduct: "PI", startTime: new DateTime(), duration: 2, measure: "render time", grouping: "1234567", reference: "123:2", failure: false, metadata: [start: "1", end: "5", error: '$10&<$5']),
                new Duration(environment: "SOTL", product: "EasyDoc", subProduct: "PI", startTime: new DateTime(), duration: 6, measure: "render time", grouping: "123", reference: "123:3", failure: false, metadata: [start: "1", end: "5", error: '$10&<$5']),
                new Duration(environment: "SOTL", product: "EasyDoc", subProduct: "PI", startTime: new DateTime(), duration: 4, measure: "end to end", grouping: "1234567", reference: "123:4", failure: false, metadata: [start: "1", end: "5", error: '$10&<$5']),
                new Duration(environment: "SOTL", product: "EasyDoc", subProduct: "PI", startTime: new DateTime(), duration: 7, measure: "end to end", grouping: "123", reference: "123:5", failure: true, metadata: [start: "1", end: "5", error: '$10&<$5']),
                new Duration(environment: "SOTL", product: "EasyDoc", subProduct: "PI", batch: true, batchType: "Other", startTime: new DateTime(), duration: 6, measure: "end to end", grouping: "1234567", reference: "123:6", failure: false, metadata: [start: "1", end: "5", error: '$10&<$5']),
                new Duration(environment: "SOTL", product: "EasyDoc", subProduct: "PI", batch: true, batchType: "Other", startTime: new DateTime(), duration: 7, measure: "end to end", grouping: "1234567", reference: "123:7", failure: false, metadata: [start: "1", end: "5", error: '$10&<$5']),
                new Duration(environment: "SOTL", product: "EasyDoc", subProduct: "ProductA", batch: true, batchType: "BatchA", startTime: new DateTime(), duration: 10, measure: "end to end", grouping: "1234567", reference: "123:8", failure: false, metadata: [start: "1", end: "5", error: '$10&<$5']),
                new Duration(environment: "SOTL", product: "EasyDoc", subProduct: "ProductA", batch: true, batchType: "BatchA", startTime: new DateTime(), duration: 29, measure: "end to end", grouping: "1234567", reference: "123:9", failure: false, metadata: [start: "1", end: "5", error: '$10&<$5']),
                new Duration(environment: "SOTL", product: "EasyDoc", batch: true, batchType: "Other", startTime: new DateTime(), duration: 8, measure: "end to end", grouping: "1234567", reference: "123:10", failure: false, metadata: [start: "1", end: "5", error: '$10&<$5'])
            ]

            new GeigerClient(new URL("http://localhost:8080/geiger")).send(durations)
        } else {
            List<Count> counts = [
                new Count(category: "target type", subCategory: "fax", period: Period.MINUTE, periodStart: new DateTime().minusMinutes(5).toDate(), productName: "EasyDoc", subProduct: "Lemon", count: 1, environment: "PROD", min: 5, max: 50, sum: 1000, failure: false),
                new Count(category: "target type", subCategory: "fax", period: Period.MINUTE, periodStart: new DateTime().minusHours(1).toDate(), productName: "EasyDoc", subProduct: "Lemon", count: 1, environment: "PROD", min: 10, max: 60, sum: 2000, failure: true),
                new Count(category: "target type", subCategory: "fax", period: Period.MINUTE, periodStart: new DateTime().minusDays(1).toDate(), productName: "EasyDoc", subProduct: "Apple", count: 1, environment: "PROD", min: 15, max: 70, sum: 3000, failure: false),
                new Count(category: "target type", subCategory: "fax", period: Period.MINUTE, periodStart: new DateTime().minusMonths(1).toDate(), productName: "EasyDoc", subProduct: "Apple", count: 1, environment: "PROD", min: 20, max: 80, sum: 4000, failure: false),
                new Count(category: "target type", subCategory: "fax", period: Period.MINUTE, periodStart: new DateTime().minusYears(1).toDate(), productName: "EasyDoc", subProduct: "Lemon", count: 1, environment: "PROD", min: 25, max: 90, sum: 5000, failure: false),

                new Count(category: "target type", subCategory: "email", period: Period.MINUTE, periodStart: new DateTime().minusMinutes(5).toDate(), productName: "EasyDoc", subProduct: "Pear", count: 1, environment: "PROD", min: 5, max: 50, sum: 1000, failure: false),
                new Count(category: "target type", subCategory: "email", period: Period.MINUTE, periodStart: new DateTime().minusHours(1).toDate(), productName: "EasyDoc", subProduct: "Pear", count: 1, environment: "PROD", min: 10, max: 60, sum: 2000, failure: true),
                new Count(category: "target type", subCategory: "email", period: Period.MINUTE, periodStart: new DateTime().minusDays(1).toDate(), productName: "EasyDoc", subProduct: "Banana", count: 1, environment: "PROD", min: 15, max: 70, sum: 3000, failure: false),
                new Count(category: "target type", subCategory: "email", period: Period.MINUTE, periodStart: new DateTime().minusMonths(1).toDate(), productName: "EasyDoc", subProduct: "Banana", count: 1, environment: "PROD", min: 20, max: 80, sum: 4000, failure: false),
                new Count(category: "target type", subCategory: "email", period: Period.MINUTE, periodStart: new DateTime().minusYears(1).toDate(), productName: "EasyDoc", subProduct: "Orange", count: 1, environment: "PROD", min: 25, max: 90, sum: 5000, failure: false),
            ]

            new GeigerClient(new URL("http://localhost:8080/geiger")).send(counts)
        }
    }

    public GeigerClient(URL url) {
        this.url = url
        client = new HttpClient()
//        batchesSent = 0
    }

    Stat send(List<Stat> stats) {
        long start = System.currentTimeMillis()
        LoaderConverter converter = createConverter(stats)
        uploadStats(converter, stats)
        logTimeTaken(System.currentTimeMillis() - start, stats.size())
        return stats.last()
    }

    void logTimeTaken(Long duration, int count) {
        if (LOG.debugEnabled) {
            totalSent += count
            totalDuration += duration
            LOG.debug("Sent $count, 1 took ${duration / count / 1000}s, $totalSent averaged ${totalDuration / totalSent / 1000}s")
        }
    }

    private void uploadStats(LoaderConverter converter, List<Stat> stats) {
//        batchesSent++
        //        cleanupClient()
        PostMethod post = new PostMethod(url.toString() + "/" + converter.loadMethod)
        try {
            post.addParameter("xml", converter.toXml(stats))
            int status = client.executeMethod(post)
            if (status != org.apache.commons.httpclient.HttpStatus.SC_OK) {
                LOG.error("Error uploading stats to $url xml to post was: $xml")
                LOG.error("status code " + post.getStatusCode())
                LOG.error(post.getResponseBodyAsString())
            } else {
                // cleanup - see http://hc.apache.org/httpclient-3.x/tutorial.html
                cleanupResponseBody(post)
            }
        } finally {
            // cleanup - see http://hc.apache.org/httpclient-3.x/tutorial.html
            post.releaseConnection()
        }
    }

//    void cleanupClient() {
    //        if(batchesSent % 100 == 0) {
    //            LOG.debug("creating a new HttpClient to free memory/resources")
    //            client = new HttpClient()
    //        }
    //    }

    private void cleanupResponseBody(PostMethod post) {
        InputStream response = post.getResponseBodyAsStream()
        BufferedReader bufferedResponse = new BufferedReader(new InputStreamReader(response));
        String line;
        while ((line = bufferedResponse.readLine()) != null) {
        }
        bufferedResponse.close();
    }

    private LoaderConverter createConverter(List<Stat> stats) {
        if (stats.first() instanceof Duration) {
            return new DurationConverter()
        } else {
            return new CountConverter()
        }
    }
}